from math_j.utils import *


def get_factors(number: int) -> list[int]:
    """
    Function for finding the (positive) factors of an integer number
    :param number: int
    :return: list[int] The factors of number
    :raises TypeError: If non-int number is passed
    """
    if number % 1 != 0:
        raise TypeError("Cannot find the factors of a non-int")

    # Ensure number is a positive int
    number = int(abs(number))

    # Take care of a few simple cases
    if number == 0:
        raise ZeroDivisionError("Zero has infinite factors")
    if number == 1:
        return [1]

    # Create output list prepopulated with 1 and number
    output = [1, number]
    # If number is greater than 2, it requires additional calculation
    if number > 2:
        for i in range(2, number // 2 + 1):
            # Skip duplicates
            if i in output:
                continue
            if number % i == 0:
                pair = number // i
                # If i is sqrt(number), only add it once
                if i == pair:
                    output.append(i)
                else:
                    output.extend((i, pair))
    output.sort()
    return output


def gcd(*numbers: int | complex) -> int:
    """
    Function for finding the gcd (Greatest Common Divisor) of a set of numbers (real and/or complex)
    :param numbers: Variable[int, complex] for complex numbers, the set of factors is the intersection
                    of the set of factors of the real and imaginary parts
    :return: int
    """
    if 1 in numbers:
        return 1
    factors = []
    for num in numbers:
        # 0 has infinite factors
        if num == 0:
            continue
        if isinstance(num, complex):
            if num.real == 0:
                factors.append(set(get_factors(int(num.imag))))
            elif num.imag == 0:
                factors.append(set(get_factors(int(num.real))))
            else:
                factors.append(set(get_factors(int(num.real))).intersection(get_factors(int(num.imag))))
        else:
            factors.append(set(get_factors(num)))
    return max(factors[0].intersection(*factors))


def product(*values: SupportsMul):
    """
    Similar to the builtin sum(), this function returns the product of the values passed in
    NOTE: works with any class that implements __mul__
    :param values: Variable[{__mul__}]
    :return: Any
    """
    out = None
    ran = False
    for val in values:
        if not ran:
            ran = True
        if out is None:
            out = val
        else:
            out = out * val
    if not ran:
        raise MathJError("product requires a list of values")
    if out is None:
        raise MathJError("product output is None: check that values.__mul__ returns a value")
    return out
