import typing
import types
import standard_j as std
from math_j.error import MathJError


# <editor-fold desc="SupportsArithmetic">
# <editor-fold desc="Standard Arithmetic">
class SupportsNeg(typing.Protocol):
    def __neg__(self): pass


class SupportsAdd(typing.Protocol):
    def __add__(self, other): pass


class SupportsSub(typing.Protocol):
    def __sub__(self, other): pass


class SupportsMul(typing.Protocol):
    def __mul__(self, other): pass


class SupportsTruediv(typing.Protocol):
    def __truediv__(self, other): pass


SupportsArithmetic = SupportsNeg | SupportsAdd | SupportsSub | SupportsMul | SupportsTruediv
# </editor-fold>


# <editor-fold desc="Reverse Arithmetic">
class SupportsRAdd(typing.Protocol):
    def __radd__(self, other): pass


class SupportsRSub(typing.Protocol):
    def __rsub__(self, other): pass


class SupportsRMul(typing.Protocol):
    def __rmul__(self, other): pass


class SupportsRTruediv(typing.Protocol):
    def __rtruediv__(self, other): pass


SupportsRArithmetic = SupportsRAdd | SupportsRSub | SupportsRMul | SupportsRTruediv
# </editor-fold>


# <editor-fold desc="In Place Arithmetic">
class SupportsIAdd(typing.Protocol):
    def __iadd__(self, other): pass


class SupportsISub(typing.Protocol):
    def __isub__(self, other): pass


class SupportsIMul(typing.Protocol):
    def __imul__(self, other): pass


class SupportsITruediv(typing.Protocol):
    def __itruediv__(self, other): pass


SupportsIArithmetic = SupportsIAdd | SupportsISub | SupportsIMul | SupportsITruediv
# </editor-fold>


SupportsFullArithmetic = SupportsArithmetic | SupportsRArithmetic | SupportsIArithmetic
# </editor-fold>


# <editor-fold desc="SupportsArithmeticExtended">
# <editor-fold desc="Standard Arithmetic Extended">
class SupportsFloordiv(typing.Protocol):
    def __floordiv__(self, other): pass


class SupportsPow(typing.Protocol):
    def __pow__(self, power, modulo=None): pass


class SupportsMod(typing.Protocol):
    def __mod__(self, other): pass


class SupportsExtendedArithmetic(SupportsFloordiv,
                                 SupportsPow,
                                 SupportsMod): pass
# </editor-fold>


# <editor-fold desc="Reverse Arithmetic Extended">
class SupportsRFloordiv(typing.Protocol):
    def __rfloordiv__(self, other): pass


class SupportsRPow(typing.Protocol):
    def __rpow__(self, power, modulo=None): pass


class SupportsRMod(typing.Protocol):
    def __rmod__(self, other): pass


SupportsRExtendedArithmetic = SupportsRFloordiv | SupportsRPow | SupportsRMod
# </editor-fold>


# <editor-fold desc="In Place Arithmetic Extended">
class SupportsIFloordiv(typing.Protocol):
    def __ifloordiv__(self, other): pass


class SupportsIPow(typing.Protocol):
    def __ipow__(self, power, modulo=None): pass


class SupportsIMod(typing.Protocol):
    def __imod__(self, other): pass


SupportsIExtendedArithmetic = SupportsIFloordiv | SupportsIPow | SupportsIMod
# </editor-fold>


SupportsFullExtendedArithmetic = SupportsExtendedArithmetic | SupportsRExtendedArithmetic | SupportsIExtendedArithmetic
# </editor-fold>


SupportsAllArithmetic = SupportsFullArithmetic | SupportsFullExtendedArithmetic


# Types
RealNumericType = int | float
NumericType = RealNumericType | complex
